import os
import glob
import argparse
from Bio import SeqIO


gc_usage = '''
===== gc example commands ===== 

BioSAK gc -i mag_1.fa
BioSAK gc -i mag_dir -x fa 

===============================
'''


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def get_gc(sequence_file):

    total_gc_num = 0
    total_len = 0
    for ctg in SeqIO.parse(sequence_file, 'fasta'):
        seq_str = str(ctg.seq)
        gc_num = (seq_str.count('G') + seq_str.count('g') + seq_str.count('C') + seq_str.count('c'))
        total_gc_num += gc_num
        total_len += len(seq_str)

    gnm_gc = float("{0:.2f}".format(total_gc_num*100/total_len))

    return gnm_gc


def gc(argument_list):

    mag_folder =    argument_list['i']
    mag_extension = argument_list['x']

    if os.path.isfile(mag_folder) is True:
        _, f_base, _ = sep_path_basename_ext(mag_folder)
        gnm_gc = get_gc(mag_folder)
        print('%s\t%s' % (f_base, gnm_gc))

    elif os.path.isdir(mag_folder) is True:

        mag_file_re   = '%s/*.%s' % (mag_folder, mag_extension)
        mag_file_list = glob.glob(mag_file_re)

        if len(mag_file_list) == 0:
            print('No file detected, program exited')
            exit()

        for each_mag in sorted(mag_file_list):
            _, f_base, _ = sep_path_basename_ext(each_mag)
            gnm_gc = get_gc(each_mag)
            print('%s\t%s' % (f_base, gnm_gc))


if __name__ == '__main__':

    gc_parser = argparse.ArgumentParser()
    gc_parser.add_argument('-i',      required=True,                          help='sequence file/folder')
    gc_parser.add_argument('-x',      required=False,                         help='file extension')
    args = vars(gc_parser.parse_args())
    gc(args)



import os
import argparse
from Bio import SeqIO
from datetime import datetime
from BioSAK.global_functions import time_format
from BioSAK.global_functions import sep_path_basename_ext


get_gene_depth_parser_usage = '''
========================== get_gene_depth example commands ==========================

# Get gene depth by contig depth, together with gbk or gff (prefered) file
BioSAK get_gene_depth -gff test.gff -ctg_depth contig_depth.txt
BioSAK get_gene_depth -gff test.gff -ctg_depth contig_depth.txt -skip_header
BioSAK get_gene_depth -gbk test.gbk -ctg_depth contig_depth.txt -skip_header

# Contig depth file format (one contig per line, tab separated)
contig_1    30.16
contig_2    26
contig_3    0.726

=====================================================================================
'''

def get_gene_depth(args):

    gbk_file =                  args['gbk']
    gff_file =                  args['gff']
    ctg_depth_file =            args['ctg_depth']
    id_column =                 args['id_column']
    depth_column =              args['depth_column']
    skip_depth_file_header =    args['skip_header']

    ################################################# check input file #################################################

    annotation_file = None
    if (gbk_file is None) and (gff_file is None):
        print(datetime.now().strftime(time_format) + 'Please provide either a gbk file or a gff file, program exited!')
        exit()

    if (gbk_file is not None) and (gff_file is not None):
        print(datetime.now().strftime(time_format) + 'Both gbk and gff file were provided, will parse annotation results from gff file')
        annotation_file = gff_file

    if (gbk_file is not None) and (gff_file is None):
        annotation_file = gbk_file

    if (gbk_file is None) and (gff_file is not None):
        annotation_file = gff_file

    # define output gene deptp file name
    annotation_file_path, annotation_file_basename, annotation_file_extension = sep_path_basename_ext(annotation_file)
    pwd_gene_depth_file = '%s/%s.depth' % (annotation_file_path, annotation_file_basename)
    if os.path.isfile(pwd_gene_depth_file) is True:
        pwd_gene_depth_file = '%s/%s.depth.txt' % (annotation_file_path, annotation_file_basename)

    ################################################ read in ctg depth #################################################

    ctg_depth_dict = {}
    line = 0
    for ctg in open(ctg_depth_file):
        ctg_split = ctg.strip().split('\t')

        if skip_depth_file_header is True:
            if line > 0:
                ctg_depth_dict[ctg_split[id_column - 1]] = float(ctg_split[depth_column - 1])
        else:
            ctg_depth_dict[ctg_split[id_column - 1]] = float(ctg_split[depth_column - 1])

        line += 1

    ########################################### get gene depth with gbk file ###########################################

    if annotation_file == gbk_file:
        gene_depth_file_handle = open(pwd_gene_depth_file, 'w')
        for seq_record in SeqIO.parse(gbk_file, 'genbank'):
            seq_id = seq_record.id
            seq_depth = ctg_depth_dict[seq_id]
            for feature in seq_record.features:
                if (feature.type != 'source') and (feature.type != 'assembly_gap'):
                    gene_id = feature.qualifiers['locus_tag'][0]
                    for_out = '%s\t%s\n' % (gene_id, seq_depth)
                    gene_depth_file_handle.write(for_out)
        gene_depth_file_handle.close()

    ########################################### get gene depth with gff file ###########################################

    if annotation_file == gff_file:
        gene_depth_file_handle = open(pwd_gene_depth_file, 'w')
        for each_line in open(gff_file):
            if not each_line.startswith('#'):
                each_line_split = each_line.strip().split('\t')
                if len(each_line_split) > 1:
                    seq_id = each_line_split[0]
                    seq_depth = ctg_depth_dict[seq_id]
                    gene_id = each_line_split[8].split(';')[0].split('ID=')[-1]
                    for_out = '%s\t%s\n' % (gene_id, seq_depth)
                    gene_depth_file_handle.write(for_out)
        gene_depth_file_handle.close()

    ###################################################### report ######################################################

    print(datetime.now().strftime(time_format) + 'Gene depth exported to %s' % pwd_gene_depth_file)
    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    get_gene_depth_parser = argparse.ArgumentParser()

    # arguments for get_gene_depth_parser
    get_gene_depth_parser.add_argument('-gbk',          required=False, default=None,        help='gbk file')
    get_gene_depth_parser.add_argument('-gff',          required=False, default=None,        help='gff file')
    get_gene_depth_parser.add_argument('-ctg_depth',    required=True,                       help='contig depth file')
    get_gene_depth_parser.add_argument('-id_column',    required=False, default=1, type=int, help='contig id column, default is 1')
    get_gene_depth_parser.add_argument('-depth_column', required=False, default=2, type=int, help='contig depth column, default is 2')
    get_gene_depth_parser.add_argument('-skip_header',  required=False, action='store_true', help='skip the 1st line in contig depth file')

    args = vars(get_gene_depth_parser.parse_args())

    get_gene_depth(args)



import os
import argparse
from time import sleep


get_abd2_mapping_usage = '''
================================= get_abd2_mapping example commands =================================

BioSAK get_abd2_mapping -r masked_ref.fa -t 36 -p Cliona1 -1 Cliona1_1.fastq -2 Cliona1_2.fastq
BioSAK get_abd2_mapping -r masked_ref.fa -t 36 -p Cliona2 -1 Cliona2_1.fastq -2 Cliona2_2.fastq
BioSAK get_abd2_mapping -r masked_ref.fa -t 36 -p Cliona3 -1 Cliona3_1.fastq -2 Cliona3_2.fastq
BioSAK get_abd2_mapping -r masked_ref.fa -t 36 -p Cliona4 -1 Cliona4_1.fastq -2 Cliona4_2.fastq

# Output files from this step: cov, rpkm and stat files

=====================================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_abd2_mapping(args):

    fq_r1             = args['1']
    fq_r2             = args['2']
    ref_seq           = args['r']
    op_dir            = args['o']
    op_prefix         = args['p']
    num_threads       = args['t']
    force_overwrite   = args['f']

    # check input files
    if os.path.isfile(ref_seq) is False:
        print('%s not found, program exited!' % ref_seq)
        exit()
    if os.path.isfile(fq_r1) is False:
        print('%s not found, program exited!' % fq_r1)
        exit()
    if os.path.isfile(fq_r2) is False:
        print('%s not found, program exited!' % fq_r2)
        exit()

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    fq1_name, _, _, _ = sep_path_basename_ext(fq_r1)
    fq2_name, _, _, _ = sep_path_basename_ext(fq_r2)
    ref_name, _, _, _ = sep_path_basename_ext(ref_seq)

    perform_decompress = False
    fq_r1_decompressed = fq_r1
    fq_r2_decompressed = fq_r2
    if fq_r1.endswith('.gz'):
        fq_r1_decompressed  = '%s/%s'               % (op_dir, fq1_name[:-3])
        fq_r2_decompressed  = '%s/%s'               % (op_dir, fq2_name[:-3])
        gunzip_cmd_r1       = 'gunzip -c %s > %s'   % (fq_r1, fq_r1_decompressed)
        gunzip_cmd_r2       = 'gunzip -c %s > %s'   % (fq_r2, fq_r2_decompressed)
        perform_decompress  = True
        os.system(gunzip_cmd_r1)
        os.system(gunzip_cmd_r2)

    elif fq_r1.endswith('.tar.gz'):
        tar_cmd_r1 = ''
        tar_cmd_r2 = ''
        print('tar to be added, program exited!')
        exit()

    # index reference sequences
    index_ref_cmd     = 'cp %s %s/; bwa index %s/%s'                                                                                                                        % (ref_seq, op_dir, op_dir, ref_name)
    bwa_cmd           = 'bwa mem -5SP -t %s %s/%s %s %s | samblaster > %s/%s.sam'                                                                                           % (num_threads, op_dir, ref_name, fq_r1_decompressed, fq_r2_decompressed, op_dir, op_prefix)
    samtools_view_cmd = 'samtools view -@ 32 -bS -h -b %s/%s.sam > %s/%s.bam'                                                                                               % (op_dir, op_prefix, op_dir, op_prefix)
    samtools_sort_cmd = 'samtools sort -@ 32 %s/%s.bam -o %s/%s.sorted.bam'                                                                                                 % (op_dir, op_prefix, op_dir, op_prefix)
    coverm_filter_cmd = 'coverm filter -b %s/%s.sorted.bam --min-read-aligned-percent 0.9 --min-read-percent-identity 0.99 --output-bam-files %s/%s.sorted_filtered.bam'    % (op_dir, op_prefix, op_dir, op_prefix)
    pileup_sh_cmd     = 'pileup.sh in=%s/%s.sorted_filtered.bam out=%s/%s.sorted_filtered.cov rpkm=%s/%s.sorted_filtered.rpkm overwrite=true'                               % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    seqkit_stat_cmd   = 'seqkit stat %s > %s/%s.stat'                                                                                                                       % (fq_r1_decompressed, op_dir, op_prefix)

    print(seqkit_stat_cmd)
    os.system(seqkit_stat_cmd)

    print(index_ref_cmd)
    os.system(index_ref_cmd)

    print(bwa_cmd)
    os.system(bwa_cmd)
    if perform_decompress is True:
        os.system('rm %s/%s' % (op_dir, fq1_name[:-3]))
        os.system('rm %s/%s' % (op_dir, fq2_name[:-3]))
    os.system('rm %s/%s'   % (op_dir, ref_name))
    os.system('rm %s/%s.*' % (op_dir, ref_name))

    print(samtools_view_cmd)
    os.system(samtools_view_cmd)
    sleep(1)
    os.system('rm %s/%s.sam' % (op_dir, op_prefix))

    print(samtools_sort_cmd)
    os.system(samtools_sort_cmd)
    sleep(1)
    os.system('rm %s/%s.bam' % (op_dir, op_prefix))

    print(coverm_filter_cmd)
    os.system(coverm_filter_cmd)
    sleep(1)
    os.system('rm %s/%s.sorted.bam' % (op_dir, op_prefix))

    print(pileup_sh_cmd)
    os.system(pileup_sh_cmd)
    sleep(1)
    os.system('rm %s/%s.sorted_filtered.bam' % (op_dir, op_prefix))

    print('Done!')


if __name__ == '__main__':

    abund_parser = argparse.ArgumentParser()
    abund_parser.add_argument('-1', required=True,                          help='fastq r1')
    abund_parser.add_argument('-2', required=True,                          help='fastq r2')
    abund_parser.add_argument('-r', required=True,                          help='masked reference sequence')
    abund_parser.add_argument('-o', required=True,                          help='output directory')
    abund_parser.add_argument('-p', required=True,                          help='output prefix')
    abund_parser.add_argument('-t', required=False, type=int, default=1,    help='number of threads, default is 1')
    abund_parser.add_argument('-f', required=False, action="store_true",    help='force overwrite')
    args = vars(abund_parser.parse_args())
    get_abd2_mapping(args)



import os
import argparse
import subprocess
import multiprocessing as mp

MetaBiosample_usage = '''
==================== MetaBiosample example commands ====================

BioSAK metaBiosample -i SAMEA5126984 -a 'isolation_source,host,geo_loc_name,lat_lon' -o op_dir
BioSAK metaBiosample -i biosamples.txt -a 'isolation_source,host,geo_loc_name' -o op_dir -t 6

Dependencies: xtract, esearch and esummary
# https://www.ncbi.nlm.nih.gov/books/NBK179288/

=======================================================================
'''

def check_executables(program_list):

    not_detected_programs = []
    for needed_program in program_list:
        if subprocess.call(['which', needed_program], stdout=open(os.devnull, 'wb')) != 0:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not detected, program exited!' % ','.join(not_detected_programs))
        exit()


def MetaBiosample(args):

    file_in         = args['i']
    att_str         = args['a']
    op_dir          = args['o']
    thread_num      = args['t']
    force_overwrite = args['f']
    execute_cmd     = args['exe']

    check_executables(['xtract', 'esearch', 'esummary'])

    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('specified output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s/tmp' % op_dir)

    attributes_to_extract = att_str.split(',')
    cmd_att_part = ''
    for each_attr_to_extract in attributes_to_extract:
        cmd_att_part += '-group Attribute -if Attribute@harmonized_name -equals "%s" -element Attribute ' % each_attr_to_extract

    accession_set = set()
    if os.path.isfile(file_in) is True:
        for each_id in open(file_in):
            accession_set.add(each_id.strip())
    else:
        accession_set.add(file_in)

    esearch_cmd_list = []
    for each_id in accession_set:
        biosample_id = each_id.strip()
        op_meta_txt = '%s/tmp/%s_metadata.txt' % (op_dir, biosample_id)
        xtract_cmd  = 'xtract -pattern DocumentSummary -element Accession -element Date -first Title %s' % cmd_att_part
        esearch_cmd = 'esearch -db biosample -query %s | esummary | %s> %s' % (biosample_id, xtract_cmd, op_meta_txt)
        esearch_cmd_list.append(esearch_cmd)

    cat_cmd = 'cat %s/tmp/*_metadata.txt > %s/combined_metadata.txt' % (op_dir, op_dir)

    cmd_txt = '%s/cmd.txt' % op_dir
    with open(cmd_txt, 'w') as cmd_txt_handle:
        cmd_txt_handle.write('\n'.join(sorted(esearch_cmd_list)) + '\n')
        cmd_txt_handle.write(cat_cmd)

    if execute_cmd is True:
        pool = mp.Pool(processes=thread_num)
        pool.map(os.system, esearch_cmd_list)
        pool.close()
        pool.join()
        os.system(cat_cmd)


if __name__ == '__main__':

    arg_parser = argparse.ArgumentParser(usage=MetaBiosample_usage)
    arg_parser.add_argument('-i',   required=True,                        help='biosample id file, one id per line')
    arg_parser.add_argument('-a',   required=True,                        help='attributes to extract')
    arg_parser.add_argument('-o',   required=True,                        help='output folder')
    arg_parser.add_argument('-t',   required=False, type=int, default=1,  help='number of threads, default: 1')
    arg_parser.add_argument('-f',   required=False, action="store_true",  help='Force overwrite existing results')
    arg_parser.add_argument('-exe', required=False, action="store_true",  help='execute cmds')
    args = vars(arg_parser.parse_args())
    MetaBiosample(args)



import os
import argparse
from distutils.spawn import find_executable


reads2bam_usage = '''
=========================== reads2bam example commands ===========================

# Dependencies: bowtie2 and samtools

BioSAK reads2bam -p sample1 -ref ref.fa -r1 R1.fq -r2 R1.fq -fq -index -t 12
BioSAK reads2bam -p sample2 -ref ref.fa -r1 R1.fa -r2 R1.fa -up unpaired.fa -index -t 12
BioSAK reads2bam -p sample3 -ref ref.fa -up unpaired.fa -index -local -t 12 -tmp
BioSAK reads2bam -p sample4 -ref ref.fa -up unpaired_R1.fa,unpaired_R2.fa -index -t 12 -tmp

==================================================================================
'''


def check_dependencies(program_list):
    not_detected_programs = []
    for needed_program in program_list:
        if find_executable(needed_program) is None:
            not_detected_programs.append(needed_program)
    if not_detected_programs != []:
        print('%s not found, program exited!' % ','.join(not_detected_programs))
        exit()


def check_input_files (file_list):
    not_detected_files = []
    for each_input_file in file_list:
        if os.path.isfile(each_input_file) is False:
            not_detected_files.append(each_input_file)
    if not_detected_files != []:
        print('%s not found, program exited!' % ','.join(not_detected_files))
        exit()


def reads2bam(args):

    op_prefix       = args['p']
    ref_seq         = args['ref']
    index_ref       = args['index']
    r1_seq          = args['r1']
    r2_seq          = args['r2']
    unpaired_seq    = args['up']
    fq_format       = args['fq']
    local_aln       = args['local']
    no_unal         = args['no_unal']
    thread_num      = args['t']
    keep_tmp        = args['tmp']

    check_dependencies(['bowtie2-build', 'bowtie2', 'samtools'])
    check_input_files([ref_seq, r1_seq, r2_seq])

    cmd_bowtie2_build = 'bowtie2-build -f %s %s --threads %s' % (ref_seq, op_prefix, thread_num)
    bowtie2_parameter = ''
    if fq_format is True:
        bowtie2_parameter += ' -q'
    else:
        bowtie2_parameter += ' -f'

    if local_aln is True:
        bowtie2_parameter += ' --local'

    if no_unal is True:
        bowtie2_parameter += ' --no-unal'

    bowtie2_parameter += (' -p %s' % thread_num)

    cmd_bowtie2 = ''
    if (r1_seq is not None) and (r2_seq is not None) and (unpaired_seq is None):
        cmd_bowtie2 = 'bowtie2 -x %s -1 %s -2 %s -S %s.sam %s'          % (op_prefix, r1_seq, r2_seq, op_prefix, bowtie2_parameter)
    elif (r1_seq is not None) and (r2_seq is not None) and (unpaired_seq is not None):
        cmd_bowtie2 = 'bowtie2 -x %s -1 %s -2 %s -U %s -S %s.sam %s'    % (op_prefix, r1_seq, r2_seq, unpaired_seq, op_prefix, bowtie2_parameter)
    elif (r1_seq is None) and (r2_seq is None) and (unpaired_seq is not None):
        cmd_bowtie2 = 'bowtie2 -x %s -U %s -S %s.sam %s'                % (op_prefix, unpaired_seq, op_prefix, bowtie2_parameter)
    else:
        print('Please check your input reads files')
        exit()

    cmd_samtools_view  = 'samtools view -bS %s.sam -o %s.bam'    % (op_prefix, op_prefix)
    cmd_samtools_sort  = 'samtools sort %s.bam -o %s_sorted.bam' % (op_prefix, op_prefix)
    cmd_samtools_index = 'samtools index %s_sorted.bam'          % op_prefix

    if index_ref is True:
        os.system(cmd_bowtie2_build)
    os.system(cmd_bowtie2)
    os.system(cmd_samtools_view)
    os.system(cmd_samtools_sort)
    os.system(cmd_samtools_index)

    if keep_tmp is False:
        os.system('rm %s.sam' % op_prefix)
        os.system('rm %s.bam' % op_prefix)


if __name__ == '__main__':

    reads2bam_parser = argparse.ArgumentParser(usage=reads2bam_usage)
    reads2bam_parser.add_argument('-p',         required=True,                          help='output prefix')
    reads2bam_parser.add_argument('-ref',       required=True,                          help='reference sequences')
    reads2bam_parser.add_argument('-index',     required=False, action="store_true",    help='index reference sequences')
    reads2bam_parser.add_argument('-r1',        required=False, default=None,           help='paired reads r1')
    reads2bam_parser.add_argument('-r2',        required=False, default=None,           help='paired reads r2')
    reads2bam_parser.add_argument('-up',        required=False, default=None,           help='unpaired reads')
    reads2bam_parser.add_argument('-fq',        required=False, action="store_true",    help='reads in fastq format')
    reads2bam_parser.add_argument('-local',     required=False, action="store_true",    help='perform local alignment')
    reads2bam_parser.add_argument('-no_unal',   required=False, action="store_true",    help='Suppress SAM records for reads that failed to align')
    reads2bam_parser.add_argument('-t',         required=False, type=int, default=1,    help='number of threads, default: 1')
    reads2bam_parser.add_argument('-tmp',       required=False, action="store_true",    help='keep temporary files')
    args = vars(reads2bam_parser.parse_args())
    reads2bam(args)




import os
import argparse


add_desc_usage = '''
====================== add_desc example commands ======================

BioSAK add_desc -i arCOG_df.txt -o arCOG_df_desc.txt -d arCOGdef.tab
BioSAK add_desc -i KEGG_df.txt -o KEGG_df_desc.txt -d ko00001.keg

=======================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def add_desc(args):

    df_in   = args['i']
    df_out  = args['o']
    db_file = args['d']

    f_path, f_base, f_ext = sep_path_basename_ext(db_file)

    # read in cog_des_txt
    description_dict = dict()
    if f_base == 'arCOGdef':
        for each_cog in open(db_file, encoding="ISO-8859-1"):
            each_cog_split = each_cog.strip().split('\t')
            cog_id = each_cog_split[0]
            cog_desc = each_cog_split[3]
            description_dict[cog_id] = cog_desc

    elif f_base == 'ko00001':
        description_dict = {}
        for each_line in open(db_file):
            if each_line[0] in ['A', 'B', 'C', 'D']:
                each_line_split = each_line.strip().split(' ')
                if each_line[0] == 'D':
                    current_D_id = each_line_split[6]
                    current_D_description = ' '.join(each_line_split[8:])
                    description_dict[current_D_id] = current_D_description
    else:
        print('Please provide the correct db file, either arCOGdef.tab or ko00001.keg')
        print('Program exited!')
        exit()

    df_out_handle = open(df_out, 'w')
    line_index = 0
    for each_line in open(df_in):
        each_line_split = each_line.strip().split('\t')
        if line_index == 0:
            list_with_desc = ['']
            for each_id in each_line_split:
                list_with_desc.append('%s__%s' % (each_id, description_dict[each_id]))
            df_out_handle.write('\t'.join(list_with_desc) + '\n')
        else:
            df_out_handle.write(each_line)
        line_index += 1
    df_out_handle.close()


if __name__ == '__main__':

    add_desc_parser = argparse.ArgumentParser()
    add_desc_parser.add_argument('-i',  required=True,  help='input file ')
    add_desc_parser.add_argument('-o',  required=True,  help='output file')
    add_desc_parser.add_argument('-d',  required=True,  help='database file')
    args = vars(add_desc_parser.parse_args())
    add_desc(args)




import argparse
from Bio import SeqIO
from BioSAK.global_functions import sep_path_basename_ext


UNITE_for_BLCA_usage = '''
========================= UNITE_for_BLCA example commands =========================

BioSAK UNITE_for_BLCA -i sh_general_release_dynamic_04.04.2024.fasta

# Input file is available from: https://doi.plutof.ut.ee/doi/10.15156/BIO/2959332

# output_file:
sh_general_release_dynamic_04.04.2024_blca_compatible.fasta
sh_general_release_dynamic_04.04.2024_blca_compatible.taxonomy

==================================================================================
'''


def UNITE_for_BLCA(args):

    db_file = args['i']

    db_file_path, db_file_basename, _ = sep_path_basename_ext(db_file)
    file_out_sequence = '%s/%s_BLCA_compatible.fasta'    % (db_file_path, db_file_basename)
    file_out_taxonomy = '%s/%s_BLCA_compatible.taxonomy' % (db_file_path, db_file_basename)

    file_out_sequence_handle = open(file_out_sequence, 'w')
    file_out_taxonomy_handle = open(file_out_taxonomy, 'w')
    for seq_record in SeqIO.parse(db_file, 'fasta'):
        seq_id_split = seq_record.id.split('|k__Fungi;')
        seq_id_new = seq_id_split[0]
        seq_id_new = seq_id_new.replace('|reps', '')
        seq_id_new = seq_id_new.replace('|refs', '')
        seq_id_new = '_'.join(seq_id_new.split('|')[1:])
        tax_str = seq_id_split[1]
        tax_str_split = tax_str.strip().split(';')
        tax_str_split_reverse = tax_str_split[::-1]
        tax_str_split_reverse.append('k__Fungi;')
        tax_str_split_reverse_str = ';'.join(tax_str_split_reverse)
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('s__', 'species:')
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('g__', 'genus:')
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('f__', 'family:')
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('o__', 'order:')
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('c__', 'class:')
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('p__', 'phylum:')
        tax_str_split_reverse_str = tax_str_split_reverse_str.replace('k__', 'superkingdom:')

        # write out to taxonomy file
        file_out_taxonomy_handle.write('%s\t%s\n' % (seq_id_new, tax_str_split_reverse_str))

        # write out to sequence file
        file_out_sequence_handle.write('>%s\n' % seq_id_new)
        file_out_sequence_handle.write('%s\n' % seq_record.seq)

    file_out_sequence_handle.close()
    file_out_taxonomy_handle.close()

    print('You may want to run makeblastdb with:')
    print('makeblastdb -in %s -dbtype nucl -parse_seqids' % file_out_sequence)


if __name__ == "__main__":

    UNITE_for_BLCA_parser = argparse.ArgumentParser()
    UNITE_for_BLCA_parser.add_argument('-i', required=True, help='sh_general_release_dynamic_04.04.2024.fasta')
    args = vars(UNITE_for_BLCA_parser.parse_args())
    UNITE_for_BLCA(args)



import os
import glob

###############################################################

input_dir    = '/Users/songweizhi/Desktop/Examples'
file_ext     = 'txt'
note_txt     = '/Users/songweizhi/Desktop/0.Basic.infor'
op_file      = '/Users/songweizhi/Desktop/op_dm.txt'
skip_1st_row = True  # specify as True or False

###############################################################


def sep_path_basename_ext(file_in):
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


note_dict = dict()
for each_line in open(note_txt):
    each_line_split = each_line.strip().split('\t')
    note_dict[each_line_split[0]] = each_line_split[1]

id_set = set()
annotation_dod = dict()
for each_file in glob.glob('%s/*.%s' % (input_dir, file_ext)):
    f_path, f_base, f_ext = sep_path_basename_ext(each_file)
    line_num_index = 1
    current_annotation_dict = dict()
    for each_line in open(each_file):
        each_line_split = each_line.strip().split('\t')
        if line_num_index == 1:
            if skip_1st_row is False:
                current_annotation_dict[each_line_split[0]] = each_line_split[1]
                id_set.add(each_line_split[0])
        else:
            current_annotation_dict[each_line_split[0]] = each_line_split[1]
            id_set.add(each_line_split[0])
        line_num_index += 1
    annotation_dod[f_base] = current_annotation_dict

id_list_sorted   = sorted([i for i in id_set])
desc_list_sorted = [note_dict.get(i, 'na') for i in id_list_sorted]
file_list_sorted = sorted(list(annotation_dod.keys()))

op_file_handle = open(op_file, 'w')
op_file_handle.write('Class\tNote\t' + '\t'.join(file_list_sorted) + '\n')
for each_id in id_list_sorted:
    id_desc = note_dict.get(each_id, 'na')
    current_id_value_list = [each_id, id_desc]
    for each_sample in file_list_sorted:
        current_dict = annotation_dod[each_sample]
        current_id_value_list.append(current_dict.get(each_id, 'na'))
    op_file_handle.write('\t'.join(current_id_value_list) + '\n')
op_file_handle.close()



import os
import argparse
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
from Bio.Graphics import GenomeDiagram
from Bio.SeqFeature import FeatureLocation
from reportlab.lib import colors
from reportlab.lib.units import cm


VisGeneFlk_usage = '''
====================== VisGeneFlk example commands ======================

BioSAK VisGeneFlk -gene gene_01 -gbk input.gbk -len 2000 -fmt pdf
BioSAK VisGeneFlk -gene gene_01 -gbk input.gbk -len 5000 -scale 300

=========================================================================
'''


def get_flanking_region(input_gbk_file, HGT_candidate, flanking_length):

    wd, gbk_file = os.path.split(input_gbk_file)
    new_gbk_file = '%s/%s_%sbp_temp.gbk' % (wd, HGT_candidate, flanking_length)
    new_gbk_final_file = '%s/%s_%sbp.gbk' % (wd, HGT_candidate, flanking_length)

    # get flanking range of candidate
    input_gbk = SeqIO.parse(input_gbk_file, "genbank")
    new_start = 0
    new_end = 0
    contig_length = 0
    for record in input_gbk:
        contig_length = len(record.seq)
        for gene in record.features:
            # get contig length
            if gene.type == 'source':
                pass
            # get new start and end points
            elif 'locus_tag' in gene.qualifiers:
                if gene.qualifiers['locus_tag'][0] == HGT_candidate:
                    # get new start point
                    new_start = gene.location.start - flanking_length
                    if new_start < 0:
                        new_start = 0
                    # get new end point
                    new_end = gene.location.end + flanking_length
                    if new_end > contig_length:
                        new_end = contig_length

    # get genes within flanking region
    keep_gene_list = []
    input_gbk = SeqIO.parse(input_gbk_file, "genbank")
    for record in input_gbk:
        for gene in record.features:
            if 'locus_tag' in gene.qualifiers:
                if (gene.location.start < new_start) and (gene.location.end >= new_start):
                    keep_gene_list.append(gene.qualifiers['locus_tag'][0])
                    new_start = gene.location.start
                elif (gene.location.start > new_start) and (gene.location.end < new_end):
                    keep_gene_list.append(gene.qualifiers['locus_tag'][0])
                elif (gene.location.start <= new_end) and (gene.location.end > new_end):
                    keep_gene_list.append(gene.qualifiers['locus_tag'][0])
                    new_end = gene.location.end

    # remove genes not in flanking region from gbk file
    input_gbk = SeqIO.parse(input_gbk_file, "genbank")
    new_gbk = open(new_gbk_file, 'w')
    for record in input_gbk:
        new_record_features = []
        for gene in record.features:
            if gene.type == 'source':
                new_record_features.append(gene)
            elif 'locus_tag' in gene.qualifiers:
                if gene.qualifiers['locus_tag'][0] in keep_gene_list:
                    new_record_features.append(gene)
        record.features = new_record_features
        SeqIO.write(record, new_gbk, 'genbank')
    new_gbk.close()

    # remove sequences not in flanking region
    new_gbk_full_length = SeqIO.parse(new_gbk_file, "genbank")
    new_gbk_final = open(new_gbk_final_file, 'w')
    for record in new_gbk_full_length:

        # get new sequence
        new_seq = record.seq[new_start:new_end]
        new_contig_length = len(new_seq)
        new_record = SeqRecord(new_seq, id=record.id, name=record.name,
                               description=record.description,
                               annotations=record.annotations)

        # get new location
        new_record_features_2 = []
        for gene in record.features:
            if gene.type == 'source':
                gene_location_new = ''
                if gene.location.strand == 1:
                    gene_location_new = FeatureLocation(0, new_contig_length, strand=+1)
                if gene.location.strand == -1:
                    gene_location_new = FeatureLocation(0, new_contig_length, strand=-1)
                gene.location = gene_location_new
                new_record_features_2.append(gene)
            elif 'locus_tag' in gene.qualifiers:

                strand_index = ''
                if gene.location.strand == 1:
                    strand_index = +1
                if gene.location.strand == -1:
                    strand_index = -1

                if gene.location.start - new_start < 0:
                    gene_location_new = FeatureLocation(0, gene.location.end - new_start, strand=strand_index)
                else:
                    gene_location_new = FeatureLocation(gene.location.start - new_start, gene.location.end - new_start, strand=strand_index)

                gene.location = gene_location_new
                new_record_features_2.append(gene)
        new_record.features = new_record_features_2
        SeqIO.write(new_record, new_gbk_final, 'genbank')

    new_gbk_final.close()
    os.system('rm %s' % new_gbk_file)


def set_contig_track_features(gene_contig, gene_to_highlight, feature_set, hide_label):

    if hide_label is False:
        show_label = True
    else:
        show_label = False

    # add features to feature set
    for feature in gene_contig.features:
        if feature.type == "CDS":

            # define label color
            if feature.qualifiers['locus_tag'][0] == gene_to_highlight:
                label_color = colors.red
                label_size = 10
            else:
                label_color = colors.black
                label_size = 10

            # strands
            color = None
            label_angle = 0
            if feature.location.strand == 1:
                label_angle = 45
                color = colors.lightblue
            elif feature.location.strand == -1:
                label_angle = -225
                color = colors.lightgreen

            # add feature
            feature_set.add_feature(feature, color=color, sigil='BIGARROW', arrowshaft_height=0.5, arrowhead_length=0.3,
                                    label=show_label, label_color=label_color, label_size=label_size, label_angle=label_angle, label_position="middle")


def VisGeneFlk(args):

    gene_id         = args['gene']
    input_gbk       = args['gbk']
    flanking_length = args['len']
    plot_scale      = args['scale']
    plot_fmt        = args['fmt']
    hide_label      = args['no_label']

    plot_wd                  = '%s_flk%s_wd'    % (gene_id, flanking_length)
    gbk_subset_located_seq   = '%s/%s.gbk'      % (plot_wd, gene_id)
    gbk_subset_flanking_gene = '%s/%s_%sbp.gbk' % (plot_wd, gene_id, flanking_length)
    plot_file                = '%s_flk%sbp.%s'  % (gene_id, flanking_length, plot_fmt)

    if os.path.isdir(plot_wd) is False:
        os.mkdir(plot_wd)
    else:
        os.system('rm -r %s' % plot_wd)
        os.mkdir(plot_wd)

    dict_value_list = []
    for seq_record in SeqIO.parse(input_gbk, 'genbank'):
        for gene_feature in seq_record.features:
            if 'locus_tag' in gene_feature.qualifiers:
                if gene_id in gene_feature.qualifiers["locus_tag"]:
                    dict_value_list.append([gene_id, int(gene_feature.location.start), int(gene_feature.location.end), gene_feature.location.strand, len(seq_record.seq)])
                    SeqIO.write(seq_record, gbk_subset_located_seq, 'genbank')
                    get_flanking_region(gbk_subset_located_seq, gene_id, flanking_length)

    # get the distance of the gene to contig ends
    gene_1_left_len = dict_value_list[0][1]
    gene_1_right_len = dict_value_list[0][4] - dict_value_list[0][2]

    # read in gbk file
    sequence_record = SeqIO.read(gbk_subset_flanking_gene, "genbank")

    # create an empty diagram
    diagram = GenomeDiagram.Diagram()
    plot_len_cm = len(sequence_record)/plot_scale

    # add tracks to diagram
    track_footnote = '%s (left %sbp, right %sbp)' % (sequence_record.name, gene_1_left_len, gene_1_right_len)
    track_footnote = sequence_record.name
    seq_track = diagram.new_track(1, name=track_footnote, greytrack=True,
                                  greytrack_labels=1, greytrack_font='Helvetica', greytrack_fontsize=12,
                                  height=0.35, start=0, end=len(sequence_record),
                                  scale=True, scale_fontsize=6, scale_ticks=1,
                                  scale_smalltick_interval=10000, scale_largetick_interval=10000)

    # create blank feature set and add gene features to it
    feature_set = seq_track.new_set(type='feature')
    set_contig_track_features(sequence_record, gene_id, feature_set, hide_label)

    # draw and export
    diagram.draw(format='linear', orientation='landscape', pagesize=(20*cm, plot_len_cm*cm), fragments=1, start=0, end=len(sequence_record))
    diagram.write(plot_file, plot_fmt)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-gene',    required=True,                          help='gene id')
    parser.add_argument('-gbk',     required=True,                          help='gbk file')
    parser.add_argument('-len',     required=True, type=int,                help='length (in bp) of flanking sequences to plot')
    parser.add_argument('-scale',   required=False, type=int, default=200,  help='scale for plotting, default: 200bp per cm')
    parser.add_argument('-fmt',     required=False, default='svg',          help='output format (svg or pdf), default: svg')
    parser.add_argument('-no_label',required=False, action='store_true',    help='output format (svg or pdf), default: svg')
    args = vars(parser.parse_args())
    VisGeneFlk(args)



'''
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/VisGeneFlk.py -gene HEOOAKGO_03443 -gbk Afipia_birgiae_34632.gbk -len 50000 -fmt pdf

'''


#!/usr/bin/env python
# coding: utf-8

import pandas as pd
import io, sys

fn1 = sys.argv[1] #GSD-20.sorted_filtered.rpkm
fn2 = sys.argv[2] #allsample.stat
fn3 = sys.argv[3] #test_out.txt

def get_sample_reads(sample_name):
    with io.open (fn2,'r', encoding = 'utf-8' ) as f:
        lines = f.readlines()
        for line in lines:
            if str(sample_name) == line.split('\t')[0].split('_1.fastq')[0]: #sample name GSD-20_1.fastq
                sample_reads = line.split('\t')[1]
                sample_reads_fnl = int(sample_reads.replace('\n','').replace(',', ''))
                return  sample_reads_fnl

if __name__ == '__main__':

    rpkm_out1 = pd.read_csv(fn1, sep = '\t',header = 4)

    rpkm_out2 = rpkm_out1[['#Name', 'Length', 'Reads']]
    rpkm_out2['bin_name'] = rpkm_out2['#Name'].str.rsplit('_', n=1, expand=True)[0] #contig name 'bin1_1', bin name 'bin1'
    rpkm_out2['Length'] =  rpkm_out2['Length'].astype(int)
    rpkm_out2['Reads'] =  rpkm_out2['Reads'].astype(int)

    rpkm_out3 = rpkm_out2.groupby(["bin_name"])[['Length','Reads']].sum()

    sample_name = str(fn1).split('.')[0]
    sample_reads_fnl = get_sample_reads(sample_name)
    rpkm_out3[sample_name+'_rpkm']=(rpkm_out3['Reads']*1000000000)/(rpkm_out3['Length']*sample_reads_fnl*2) #calulate RPKM
    rpkm_out3.to_csv(fn3,sep='\t')





import os
import glob
import argparse
from Bio import SeqIO


def get_ref_taxonomy(args):

    fsa_folder =    args['i']
    file_out =      args['o']
    taxonomy_file = args['t']

    fsa_file_re = '%s/*.hitdb.fsa' % fsa_folder
    fsa_file_list = [os.path.basename(file_name) for file_name in glob.glob(fsa_file_re)]

    # store taxonomy info in dict
    taxonomy_dict = {}
    for ref in open(taxonomy_file):
        ref_split = ref.strip().split('\t')
        ref_id = ref_split[0]
        ref_taxon = ref_split[1]
        ref_taxon_split_no_rank = [i.split(':')[1] for i in ref_taxon.split(';')[:-1]]
        ref_taxon_no_rank = ';'.join(ref_taxon_split_no_rank[::-1])
        taxonomy_dict[ref_id] = ref_taxon_no_rank

    file_out_handle = open(file_out, 'w')
    for fsa_file in fsa_file_list:
        pwd_fas_file = '%s/%s' % (fsa_folder, fsa_file)
        for seq in SeqIO.parse(pwd_fas_file, 'fasta'):
            if seq.id in taxonomy_dict:
                file_out_handle.write('%s\t%s\t%s\n' % (fsa_file[:-10], seq.id, taxonomy_dict[seq.id]))


if __name__ == '__main__':

    get_best_hit_parser = argparse.ArgumentParser()

    # arguments for COG_parser
    get_best_hit_parser.add_argument('-i',   required=True,  help='folder holds .hitdb.fsa files')
    get_best_hit_parser.add_argument('-t',   required=True,  help='taxonomy file')
    get_best_hit_parser.add_argument('-o',   required=True,  help='output file')

    args = vars(get_best_hit_parser.parse_args())

    get_ref_taxonomy(args)




fsa_folder = '/Users/songweizhi/Desktop/OTU_table_GTDB_BLCA'
taxonomy_file = '/Users/songweizhi/Desktop/OTU_table_GTDB_BLCA/00_DataNeeded/BLCA/db_GTDB_SSU/GTDB_bac120_ar122_ssu_r89_BLCAparsed.taxonomy'
file_out = '/Users/songweizhi/Desktop/OTU_table_GTDB_BLCA.txt'



import os
import shutil
from Bio import SeqIO
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

time_format = '[%Y-%m-%d %H:%M:%S] '


def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        return False


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)

    os.mkdir(folder_to_create)


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_ext = os.path.splitext(file_name)

    return file_path, file_basename, file_ext


def get_no_hidden_folder_list(wd):
    folder_list = []
    for each_folder in os.listdir(wd):
        if not each_folder.startswith('.'):
            folder_list.append(each_folder)
    return folder_list


def unique_list_elements(list_input):

    list_output = []
    for each_element in list_input:
        if each_element not in list_output:
            list_output.append(each_element)

    return list_output


def ctg_depth_and_gbk_to_gene_depth(ctg_depth_file, gbk_file, skip_depth_file_header, gene_depth_file_folder):

    gbk_file_path, gbk_file_basename, gbk_file_extension = sep_path_basename_ext(gbk_file)
    pwd_depth_file = '%s/%s.depth' % (gene_depth_file_folder, gbk_file_basename)

    # read in depth
    ctg_depth_dict = {}
    line = 0
    for ctg in open(ctg_depth_file):

        ctg_split = ctg.strip().split('\t')

        if skip_depth_file_header is True:
            if line > 0:
                ctg_depth_dict[ctg_split[0]] = float(ctg_split[1])
        else:
            ctg_depth_dict[ctg_split[0]] = float(ctg_split[1])

        line += 1

    # get gene depth
    gene_depth_file_handle = open(pwd_depth_file, 'w')
    gene_depth_file_handle.write('Gene\tDepth\n')
    for seq_record in SeqIO.parse(gbk_file, 'genbank'):

        seq_id = seq_record.id
        seq_depth = ctg_depth_dict[seq_id]

        for feature in seq_record.features:
            if feature.type == 'CDS':
                gene_id = feature.qualifiers['locus_tag'][0]
                for_out = '%s\t%s\n' % (gene_id, seq_depth)
                gene_depth_file_handle.write(for_out)

    gene_depth_file_handle.close()


def barh_plotter(num_list, label_list, query_seq_num, query_ko_NA, fig_width, fig_height, plot_file):

    fig, ax = plt.subplots()
    fig.set_size_inches(fig_width, fig_height)

    y_pos = range(len(num_list))
    ax.barh(y_pos, num_list, height=0.8, align='center', alpha=0.2, linewidth=0)
    ax.set_yticks([])  # not show yticks
    ax.invert_xaxis()  # line up bar on right
    ax.invert_yaxis()  # put first number on top
    ax.axis('tight')   # remove extra spaces at the top and bottom, equal to: ax.margins(0, 0)
    # ax.margins(0, 0.01) # customize space percentage

    ax.set_xlabel('Number of gene')
    ax.set_title('Query genes number: %s, genes without KO: %s' % (query_seq_num, query_ko_NA))

    ax2 = ax.twinx()
    ax2.set_ylim(ax.get_ylim())
    ax2.set_yticks(y_pos)
    ax2.set_yticklabels(label_list)

    plt.tight_layout()
    plt.savefig(plot_file, dpi=300)
    plt.close()
    plt.clf()


def AnnotateNorm(file_in, skip_header, value_column, Divisor_value, file_out, file_out_header):

    file_out_handle = open(file_out, 'w')
    file_out_handle.write(file_out_header)
    line_num = 0
    for each_line in open(file_in):

        each_line_split = each_line.strip().split('\t')
        value_str = each_line_split[value_column - 1]

        if (skip_header is True and line_num > 0) or (skip_header is False):

            value_pct = 0
            if Divisor_value != 0:
                value_pct = float(value_str) * 100 / Divisor_value
            each_line_split[value_column - 1] = str(float("{0:.2f}".format(value_pct)))
            file_out_handle.write('%s\n' % '\t'.join(each_line_split))

        line_num += 1

    file_out_handle.close()


def get_gene_list_TotalDepth(gene_list, gene_to_depth_dict):

    total_depth = 0
    for gene in gene_list:
        gene_depth = gene_to_depth_dict[gene]
        total_depth += gene_depth

    return total_depth




import argparse


def get_reads_from_sam(args):

    sam_file    = args['sam']
    output_file = args['out']
    ctgs_file   = args['ctgs']
    option      = args['option']

    # get contig list
    ctg_list = []
    for each_ctg in open(ctgs_file):
        each_ctg = each_ctg.strip()
        ctg_list.append(each_ctg)

    # export reads
    output = open(output_file, 'w')
    for each in open(sam_file):
        if not each.startswith('@'):
            each_split = each.strip().split('\t')
            query_name = each_split[0]
            ref_name = each_split[2]
            query_seq = each_split[9]

            if option == 1:
                if ref_name in ctg_list:
                    output.write('>%s\n' % query_name)
                    output.write('%s\n' % query_seq)

            if option == 0:
                if ref_name not in ctg_list:
                    output.write('>%s\n' % query_name)
                    output.write('%s\n' % query_seq)

    output.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-sam',     required=True,              help='Input sam file')
    parser.add_argument('-ctgs',    required=True,              help='Contig list')
    parser.add_argument('-option',  required=True, type=int,    help="Specify '1' to get reads mapped to provided contigs, '0' to get reads not mapped to provided contigs")
    parser.add_argument('-out',     required=True,              help='Output fasta file')
    args = vars(parser.parse_args())
    get_reads_from_sam(args)



import argparse
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


boxplot_usage = '''
===================== boxplot example commands =====================

python3 boxplot_with_dots.py -i demo_boxplot.txt -o boxplot.pdf
python3 boxplot_with_dots.py -i demo_boxplot.txt -o boxplot.pdf -d

# format of input file (tab separated, no row name)
Apple	Banana	Orange
1	4	6
2	5	7
3	6	8
4	7	9

====================================================================
'''


def boxplot_with_dots(args):

    data_file   = args['i']
    add_dots    = args['d']
    output_plot = args['o']

    # read in file
    data_dict = dict()
    col_index = dict()
    line_num_index = 0
    for each_line in open(data_file):
        line_num_index += 1
        line_split = each_line.strip().split('\t')
        if line_num_index == 1:
            col_index = {key: i for key, i in enumerate(line_split)}
            data_dict = {i: [] for key, i in enumerate(line_split)}
        else:
            element_pos = 0
            for each_element in line_split:
                grp = col_index[element_pos]
                print('%s\t%s\t%s' % (grp, element_pos, each_element))
                data_dict[grp].append(float(each_element))
                element_pos += 1

    # get label
    name_list = sorted(list(data_dict.keys()))
    num_lol = [data_dict[i] for i in name_list]

    # plot
    data = [np.array(i) for i in num_lol]
    box_plot = plt.boxplot(data, labels=name_list, patch_artist=True, whiskerprops=dict(color='black', linewidth=1), capprops=dict(color='black'))

    # set the color pf box
    for box in box_plot['boxes']:
        box.set(linewidth=1)
        box.set_facecolor('white')

    # add dots, https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.pyplot.plot.html
    if add_dots is True:
        col_index = 1
        for num_arrary in data:
            plt.plot(np.random.normal(col_index, 0.02, len(num_arrary)), num_arrary, '.', alpha=0.8, color='orange', markersize=6, markeredgewidth=0)
            col_index += 1

    # write out
    plt.tight_layout()
    plt.savefig(output_plot, bbox_inches='tight', dpi=300)
    plt.close()


if __name__ == '__main__':

    boxplot_parser = argparse.ArgumentParser()
    boxplot_parser.add_argument('-i',   required=True,                          help='input files')
    boxplot_parser.add_argument('-d',   required=False, action="store_true",    help='show dots')
    boxplot_parser.add_argument('-o',   required=True,                          help='output plot')
    args = vars(boxplot_parser.parse_args())
    boxplot_with_dots(args)



import os
import re
import sys
import collections
from Bio import SeqIO


"""
Convert data from Genbank format to GFF. 

Usage: 
python gbk_to_gff.py in.gbk > out.gff 

Requirements:
    BioPython:- http://biopython.org/
    helper.py:- https://github.com/vipints/GFFtools-GX/blob/master/helper.py

Copyright (C) 
    2009-2012 Friedrich Miescher Laboratory of the Max Planck Society, Tubingen, Germany.
    2012-2015 Memorial Sloan Kettering Cancer Center New York City, USA.
"""

def feature_table(chr_id, source, orient, genes, transcripts, cds, exons, unk):
    """
    Write the feature information
    """
    for gname, ginfo in genes.items():
        line = [str(chr_id),
                'gbk2gff',
                ginfo[3],
                str(ginfo[0]),
                str(ginfo[1]),
                '.',
                ginfo[2],
                '.',
                'ID=%s;Name=%s' % (str(gname), str(gname))]
        sys.stdout.write('\t'.join(line) + "\n")
        ## construct the transcript line is not defined in the original file
        t_line = [str(chr_id), 'gbk2gff', source, '0', '1', '.', ginfo[2], '.']

        if not transcripts:
            t_line.append('ID=Transcript:%s;Parent=%s' % (str(gname), str(gname)))

            if exons:  ## get the entire transcript region  from the defined feature
                t_line[3] = str(exons[gname][0][0])
                t_line[4] = str(exons[gname][0][-1])
            elif cds:
                t_line[3] = str(cds[gname][0][0])
                t_line[4] = str(cds[gname][0][-1])

            if not cds:
                t_line[2] = 'transcript'
            else:
                t_line[2] = 'mRNA'
            sys.stdout.write('\t'.join(t_line) + "\n")

            if exons:
                exon_line_print(t_line, exons[gname], 'Transcript:' + str(gname), 'exon')

            if cds:
                exon_line_print(t_line, cds[gname], 'Transcript:' + str(gname), 'CDS')
                if not exons:
                    exon_line_print(t_line, cds[gname], 'Transcript:' + str(gname), 'exon')

        else:  ## transcript is defined
            for idx in transcripts[gname]:
                t_line[2] = idx[3]
                t_line[3] = str(idx[0])
                t_line[4] = str(idx[1])
                t_line.append('ID=' + str(idx[2]) + ';Parent=' + str(gname))
                sys.stdout.write('\t'.join(t_line) + "\n")

                ## feature line print call
                if exons:
                    exon_line_print(t_line, exons[gname], str(idx[2]), 'exon')
                if cds:
                    exon_line_print(t_line, cds[gname], str(idx[2]), 'CDS')
                    if not exons:
                        exon_line_print(t_line, cds[gname], str(idx[2]), 'exon')

    if len(genes) == 0:  ## feature entry with fragment information

        line = [str(chr_id), 'gbk2gff', source, 0, 1, '.', orient, '.']
        fStart = fStop = None

        for eid, ex in cds.items():
            fStart = ex[0][0]
            fStop = ex[0][-1]

        for eid, ex in exons.items():
            fStart = ex[0][0]
            fStop = ex[0][-1]

        if fStart or fStart:

            line[2] = 'gene'
            line[3] = str(fStart)
            line[4] = str(fStop)
            line.append('ID=Unknown_Gene_' + str(unk) + ';Name=Unknown_Gene_' + str(unk))
            sys.stdout.write('\t'.join(line) + "\n")

            if not cds:
                line[2] = 'transcript'
            else:
                line[2] = 'mRNA'

            line[8] = 'ID=Unknown_Transcript_' + str(unk) + ';Parent=Unknown_Gene_' + str(unk)
            sys.stdout.write('\t'.join(line) + "\n")

            if exons:
                exon_line_print(line, cds[None], 'Unknown_Transcript_' + str(unk), 'exon')

            if cds:
                exon_line_print(line, cds[None], 'Unknown_Transcript_' + str(unk), 'CDS')
                if not exons:
                    exon_line_print(line, cds[None], 'Unknown_Transcript_' + str(unk), 'exon')

            unk += 1

    return unk


def exon_line_print(temp_line, trx_exons, parent, ftype):
    """
    Print the EXON feature line
    """
    for ex in trx_exons:
        temp_line[2] = ftype
        temp_line[3] = str(ex[0])
        temp_line[4] = str(ex[1])
        temp_line[8] = 'Parent=%s' % parent
        sys.stdout.write('\t'.join(temp_line) + "\n")


def gbk_parse(fname):
    """
    Extract genome annotation recods from genbank format

    @args fname: gbk file name
    @type fname: str
    """
    unk = 1
    for record in SeqIO.parse(gbkfname, "genbank"):
        gene_tags = dict()
        tx_tags = collections.defaultdict(list)
        exon = collections.defaultdict(list)
        cds = collections.defaultdict(list)
        mol_type, chr_id = None, None

        for rec in record.features:

            if rec.type == 'source':
                try:
                    mol_type = rec.qualifiers['mol_type'][0]
                except:
                    mol_type = '.'
                    pass
                try:
                    chr_id = rec.qualifiers['chromosome'][0]
                except:
                    chr_id = record.name
                continue

            strand = '-'
            strand = '+' if rec.strand > 0 else strand

            fid = None
            try:
                fid = rec.qualifiers['gene'][0]
            except:
                pass

            transcript_id = None
            try:
                transcript_id = rec.qualifiers['transcript_id'][0]
            except:
                pass

            if re.search(r'gene', rec.type):
                gene_tags[fid] = (rec.location._start.position + 1,
                                  rec.location._end.position,
                                  strand,
                                  rec.type
                                  )
            elif rec.type == 'exon':
                exon[fid].append((rec.location._start.position + 1,
                                  rec.location._end.position))
            elif rec.type == 'CDS':
                cds[fid].append((rec.location._start.position + 1,
                                 rec.location._end.position))
            else:
                # get all transcripts
                if transcript_id:
                    tx_tags[fid].append((rec.location._start.position + 1,
                                         rec.location._end.position,
                                         transcript_id,
                                         rec.type))
        # record extracted, generate feature table
        unk = feature_table(chr_id, mol_type, strand, gene_tags, tx_tags, cds, exon, unk)


if __name__ == '__main__':

    try:
        gbkfname = sys.argv[1]
    except:
        print
        __doc__
        sys.exit(-1)

    ## extract gbk records
    gbk_parse(gbkfname)



import os
import argparse


VisBlastOp_usage = '''
====================== VisBlastOp example commands ======================

BioSAK VisBlastOp -h

=========================================================================
'''


def VisBlastOp(args):
    print('To be added!')


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-gene', required=True, help='gene id')
    args = vars(parser.parse_args())
    VisBlastOp(args)



import os
import time
import argparse
from datetime import datetime


KeepRemovingTmp_usage = '''
============== KeepRemovingTmp example command ==============

BioSAK KeepRemovingTmp -i gnm.fa -o ctg_id.txt
BioSAK KeepRemovingTmp -i reads.fastq -o reads_id.txt -fq

=============================================================
'''


def get_time_since_last_modification(target_folder_or_file):

    last_modified_time = os.path.getmtime(target_folder_or_file)
    current_time = time.time()
    tslm_sec  = current_time - last_modified_time
    tslm_min  = tslm_sec / 60
    tslm_hour = tslm_sec / (60 * 60)
    tslm_day  = tslm_sec / (60 * 60 * 24)
    tslm_sec  = float("{0:.2f}".format(tslm_sec))
    tslm_min  = float("{0:.2f}".format(tslm_min))
    tslm_hour = float("{0:.2f}".format(tslm_hour))
    tslm_day  = float("{0:.2f}".format(tslm_day))

    return tslm_sec, tslm_min, tslm_hour, tslm_day


def KeepRemovingTmp(args):

    tmp_dir  = args['d']
    time_str = args['l']

    time_format = '[%Y-%m-%d %H:%M:%S]'

    time_unit = time_str[-1]
    if time_unit not in ['s', 'm', 'h', 'd']:
        print('Invalid time format, program exited!')
        print('Here are a few examples: 30s, 30m, 6h, 5d')
        exit()

    time_value = float(time_str[:-1])

    time_len_sec = None
    if time_unit == 's':
        time_len_sec = time_value
    elif time_unit == 'm':
        time_len_sec = time_value*60
    elif time_unit == 'h':
        time_len_sec = time_value*60*60
    elif time_unit == 'd':
        time_len_sec = time_value*60*60*24

    sub_dir_list = next(os.walk(tmp_dir))[1]

    for each_sub_dir in sub_dir_list:
        pwd_sub_dir = '%s/%s' % (tmp_dir, each_sub_dir)
        subdir_tslm_sec, subdir_tslm_min, subdir_tslm_hour, subdir_tslm_day = get_time_since_last_modification(pwd_sub_dir)
        if subdir_tslm_sec >= time_len_sec:

            if time_unit == 's':
                print('%s Removing %s\t(last modified %ss ago)' % (datetime.now().strftime(time_format), each_sub_dir, subdir_tslm_sec))
            elif time_unit == 'm':
                print('%s Removing %s\t(last modified %sm ago)' % (datetime.now().strftime(time_format), each_sub_dir, subdir_tslm_min))
            elif time_unit == 'h':
                print('%s Removing %s\t(last modified %sh ago)' % (datetime.now().strftime(time_format), each_sub_dir, subdir_tslm_hour))
            elif time_unit == 'd':
                print('%s Removing %s\t(last modified %sd ago)' % (datetime.now().strftime(time_format), each_sub_dir, subdir_tslm_day))

            os.system('rm -r %s' % pwd_sub_dir)

    print('%s Done!' % datetime.now().strftime(time_format), )


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-d', required=True, help='tmp dir')
    parser.add_argument('-l', required=True, help='time length, e.g., 10s, 30m, 6h or 5d')
    args = vars(parser.parse_args())
    KeepRemovingTmp(args)


'''
python3 /home-user/wzsong/Scripts/rm_old_folder.py -d /home-user/wzsong/Japonicum/gapseq_metacyc/gapseq_tmp_dir -l 30m
'''



import os


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


########################################################################################################################

fasta_file = 'symbionts_dRep99_sponge_coral_66.fna'


########################################################################################################################

_, _, f_base, _ = sep_path_basename_ext(fasta_file)

# get rRNA regions
shell_cmd_1_metaxa2_ssu_cmd     = 'metaxa2 --plus --mode m --cpu 12 --multi_thread T --table T -g ssu --not_found T -i %s -o %s.metaxa2_ssu'    % (fasta_file, f_base)
shell_cmd_2_metaxa2_lsu_cmd     = 'metaxa2 --plus --mode m --cpu 12 --multi_thread T --table T -g lsu --not_found T -i %s -o %s.metaxa2_lsu'    % (fasta_file, f_base)
shell_cmd_3                     = 'cut -f 1,9,10 %s.metaxa2_ssu.extraction.results > %s.masked_metaxa_ssu.bed'                                  % (f_base, f_base)
shell_cmd_4                     = 'cut -f 1,9,10 %s.metaxa2_lsu.extraction.results > %s.masked_metaxa_lsu.bed'                                  % (f_base, f_base)
shell_cmd_5                     = 'cat %s.masked_metaxa_ssu.bed %s.masked_metaxa_lsu.bed > %s.masked_metaxa.bed'                                % (f_base, f_base, f_base)
shell_cmd_6_barrnap_bac_cmd     = 'barrnap --kingdom bac --threads 12 --reject 0.3 %s > %s.barrnap_bac.gff'                                     % (fasta_file, f_base)
shell_cmd_7_barrnap_arc_cmd     = 'barrnap --kingdom arc --threads 12 --reject 0.3 %s > %s.barrnap_arc.gff'                                     % (fasta_file, f_base)
shell_cmd_8_barrnap_euk_cmd     = 'barrnap --kingdom euk --threads 12 --reject 0.3 %s > %s.barrnap_euk.gff'                                     % (fasta_file, f_base)
shell_cmd_9                     = 'cut -f 1,4,5 %s.barrnap_bac.gff > %s.masked_barrnap_bac.bed'                                                 % (f_base, f_base)
shell_cmd_10                    = 'cut -f 1,4,5 %s.barrnap_arc.gff > %s.masked_barrnap_arc.bed'                                                 % (f_base, f_base)
shell_cmd_11                    = 'cut -f 1,4,5 %s.barrnap_euk.gff > %s.masked_barrnap_euk.bed'                                                 % (f_base, f_base)
shell_cmd_12                    = 'cat %s.masked_barrnap_bac.bed %s.masked_barrnap_arc.bed %s.masked_barrnap_euk.bed > %s.masked_barrnap.bed'   % (f_base, f_base, f_base, f_base)
shell_cmd_13                    = 'cat %s.masked_barrnap.bed %s.masked_metaxa.bed > %s.masked_rrna.bed'                                         % (f_base, f_base, f_base)

# get tRNA regions
shell_cmd_14_trna_scan_arc_cmd  = 'tRNAscan-SE -A -b %s.ar.bedformat --thread 36 %s'                                                            % (f_base, fasta_file)
shell_cmd_15_trna_scan_bac_cmd  = 'tRNAscan-SE -B -b %s.bac.bedformat --thread 36 %s'                                                           % (f_base, fasta_file)
shell_cmd_16                    = 'cat %s.ar.bedformat %s.bac.bedformat > %s.bedformat'                                                         % (f_base, f_base, f_base)
shell_cmd_17                    = 'cut -f 1,2,3 %s.bedformat > %s.masked_trnascan.bed'                                                          % (f_base, f_base)

# find duplicated areas
shell_cmd_18                    = 'dustmasker -in %s -out %s.lowcom.out -outfmt acclist'                                                        % (fasta_file, f_base)
shell_cmd_19                    = "cut -d '>' -f 2 %s.lowcom.out > %s.masked_dustmasker.bed"                                                    % (f_base, f_base)

# cover the above area with NNNN
shell_cmd_20                    = "cat %s.masked_rrna.bed %s.masked_trnascan.bed %s.masked_dustmasker.bed > %s.mask.bed"                        % (f_base, f_base, f_base, f_base)
shell_cmd_21                    = "awk -F'\t' '$2>=0' %s.mask.bed > %s.mask_0.bed"                                                              % (f_base, f_base)
shell_cmd_22                    = "bedtools maskfasta -fi %s -bed %s.mask_0.bed -fo %s.masked.fasta"                                            % (fasta_file, f_base, f_base)

# run cmds
os.system(shell_cmd_1_metaxa2_ssu_cmd)
os.system(shell_cmd_2_metaxa2_lsu_cmd)
os.system(shell_cmd_3)
os.system(shell_cmd_4)
os.system(shell_cmd_5)
os.system(shell_cmd_6_barrnap_bac_cmd)
os.system(shell_cmd_7_barrnap_arc_cmd)
os.system(shell_cmd_8_barrnap_euk_cmd)
os.system(shell_cmd_9)
os.system(shell_cmd_10)
os.system(shell_cmd_11)
os.system(shell_cmd_12)
os.system(shell_cmd_13)
os.system(shell_cmd_14_trna_scan_arc_cmd)
os.system(shell_cmd_15_trna_scan_bac_cmd)
os.system(shell_cmd_16)
os.system(shell_cmd_17)
os.system(shell_cmd_18)
os.system(shell_cmd_19)
os.system(shell_cmd_20)
os.system(shell_cmd_21)
os.system(shell_cmd_22)



import os
import argparse


trim_usage = '''
========================== trim example commands ==========================

BioSAK trim -a TruSeq3-PE-2.fa -1 R1.fastq -2 R2.fastq -x fastq
BioSAK trim -a TruSeq3-PE-2.fa -1 R1.fastq.gz -2 R2.fastq.gz -x fastq.gz

===========================================================================
'''


def trim(args):

    fastq_r1        = args['1']
    fastq_r2        = args['2']
    file_ext        = args['x']
    adapter_file    = args['a']
    leading         = args['leading']
    trailing        = args['trailing']
    crop            = args['crop']
    headcrop        = args['headcrop']
    swl             = args['swl']
    swq             = args['swq']
    minlen          = args['minlen']

    r1_base = fastq_r1[:-(len(file_ext) + 1)]
    r2_base = fastq_r2[:-(len(file_ext) + 1)]

    r1_p  = '%s_P.%s'   % (r1_base, file_ext)
    r1_up = '%s_UP.%s'  % (r1_base, file_ext)
    r2_p  = '%s_P.%s'   % (r2_base, file_ext)
    r2_up = '%s_UP.%s'  % (r2_base, file_ext)

    trimmomatic_cmd = 'trimmomatic PE %s %s %s %s %s %s ILLUMINACLIP:%s:2:30:10 LEADING:%s TRAILING:%s CROP:%s HEADCROP:%s SLIDINGWINDOW:%s:%s MINLEN:%s' % (fastq_r1, fastq_r2, r1_p, r1_up, r2_p, r2_up, adapter_file, leading, trailing, crop, headcrop, swl, swq, minlen)
    fastqc_cmd      = 'fastqc %s %s' % (r1_p, r2_p)

    # run trimmomatic
    print(trimmomatic_cmd)
    os.system(trimmomatic_cmd)

    # run fastqc
    print(fastqc_cmd)
    os.system(fastqc_cmd)

    print('Done!')


if __name__ == '__main__':
    trim_parser = argparse.ArgumentParser(usage=trim_usage)
    trim_parser.add_argument('-1',              required=True,                          help='fastq R1')
    trim_parser.add_argument('-2',              required=True,                          help='fastq R2')
    trim_parser.add_argument('-x',              required=True,                          help='file extension, e.g., fastq or fastq.gz')
    trim_parser.add_argument('-a',              required=True,                          help='adapter file, e.g., TruSeq3-PE-2.fa')
    trim_parser.add_argument('-leading',        required=False, type=int, default=25,   help='leading, default is 25')
    trim_parser.add_argument('-trailing',       required=False, type=int, default=25,   help='trailing, default is 25')
    trim_parser.add_argument('-crop',           required=False, type=int, default=145,  help='crop, default is 145')
    trim_parser.add_argument('-headcrop',       required=False, type=int, default=5,    help='headcrop, default is 5')
    trim_parser.add_argument('-swl',            required=False, type=int, default=5,    help='slidingwindow length, default is 5')
    trim_parser.add_argument('-swq',            required=False, type=int, default=25,   help='slidingwindow q-value, default is 25')
    trim_parser.add_argument('-minlen',         required=False, type=int, default=36,   help='minlen, default is 36')
    args = vars(trim_parser.parse_args())
    trim(args)



import os
import argparse


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


parser = argparse.ArgumentParser()
parser.add_argument('-i', required=False, type=int, help='input file, -o2')
args = vars(parser.parse_args())

txt_in      = args['i']

f_path, f_base, f_ext = sep_path_basename_ext(txt_in)

txt_out     = '%s/%s_krona.txt'     % (f_path, f_base)
krona_out   = '%s/%s_krona.html'    % (f_path, f_base)

tax_count_dict = dict()
for each in open(txt_in):
    each_split = each.strip().split(',')
    tax_str = '\t'.join(each_split[1:]).replace(' ', '_')
    print(tax_str)
    if tax_str not in tax_count_dict:
        tax_count_dict[tax_str] = 1
    else:
        tax_count_dict[tax_str] += 1

txt_out_handle = open(txt_out, 'w')
for each in tax_count_dict:
    txt_out_handle.write('%s\t%s\n' % (tax_count_dict[each], each))
txt_out_handle.close()

ktImportText_cmd = 'ktImportText %s -o %s' % (txt_out, krona_out)
os.system(ktImportText_cmd)



import os
import argparse


sankey_usage = '''
================== sankey example commands ==================

BioSAK sankey -p demo_sankey -i sankey_matrix.txt
BioSAK sankey -p demo_sankey -i sankey_matrix.txt -m 3

# Requires R and R packages optparse, tools and googleVis.
# Input file need to be tab separated, non column header.

=============================================================
'''


def unique_list_elements(list_input):

    list_output = []
    for each_element in list_input:
        if each_element not in list_output:
            list_output.append(each_element)

    return list_output


def sankey(args):

    input_datamatrix_txt  = args['i']
    output_prefix         = args['p']
    plot_width            = args['x']
    plot_height           = args['y']
    min_link_value        = args['m']

    pwd_self              = os.path.realpath(__file__)
    self_path             = '/'.join(pwd_self.split('/')[:-1])
    pwd_get_sankey_plot_R = '%s/sankey.R'   % self_path
    output_file_txt       = '%s_sankey.txt' % output_prefix

    paired_taxon_list_all = []
    genome_number = 0
    max_col_num = 0
    null_col_list = []
    line_index = 1
    for each_line in open(input_datamatrix_txt):
        each_line_split = each_line.strip().split('\t')
        genome_number += 1

        if '' in each_line_split:
            null_col_list.append(str(line_index))

        if (len(each_line_split) - 1) > max_col_num:
            max_col_num = (len(each_line_split) - 1)

        n = 0
        while n < (len(each_line_split) - 1):
            paired_taxon_list_all.append('%s,%s' % (each_line_split[n], each_line_split[n+1]))
            n += 1

        line_index += 1

    if len(null_col_list) > 0:
        print('Please make sure that there are no empty cells in data matrix, program exited!')
        print('Pay attention to the following rows: %s' % ','.join(null_col_list))
        exit()

    paired_taxon_list_uniq = unique_list_elements(paired_taxon_list_all)
    paired_taxon_list_uniq_count_dict = {}
    for each_key in paired_taxon_list_uniq:
        paired_taxon_list_uniq_count_dict[each_key] = paired_taxon_list_all.count(each_key)

    output_file_handle = open(output_file_txt, 'w')
    output_file_handle.write('C1,C2,Number\n')
    for each_genome_taxon in paired_taxon_list_uniq_count_dict:
        link_value = paired_taxon_list_uniq_count_dict[each_genome_taxon]
        for_out = '%s,%s\n' % (each_genome_taxon, link_value)
        if link_value >= min_link_value:
            output_file_handle.write(for_out)
    output_file_handle.close()

    # run R script
    if plot_width is None:
        if max_col_num <= 5:
            plot_width = (max_col_num - 1) * 300
        else:
            plot_width = (max_col_num - 1) * 250

    if plot_height is None:
        plot_height = genome_number * 2
        if plot_height < 600:
            plot_height = 600

    print('Rscript sankey.R -f %s -x %s -y %s' % (output_file_txt, plot_width, plot_height))
    os.system('Rscript %s -f %s -x %s -y %s'   % (pwd_get_sankey_plot_R, output_file_txt, plot_width, plot_height))


if __name__ == '__main__':

    sankey_parser = argparse.ArgumentParser(usage=sankey_usage)
    sankey_parser.add_argument('-i', required=True,                       help='input datamatrix')
    sankey_parser.add_argument('-p', required=True,                       help='output prefix')
    sankey_parser.add_argument('-m', required=False, type=int, default=1, help='Minimal value to plot, default is 1')
    sankey_parser.add_argument('-x', required=False, type=int,            help='plot width')
    sankey_parser.add_argument('-y', required=False, type=int,            help='plot height')
    args = vars(sankey_parser.parse_args())
    sankey(args)



import os
import glob
import math
import random
import argparse
import numpy as np
import seaborn as sns
from Bio import SeqIO
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.patches import Patch


Plot_MAG_parser_usage = '''
================= Plot_MAG example commands =================

BioSAK Plot_MAG -i MAG_dir -x fa -d ctg_depth.txt 
BioSAK Plot_MAG -i MAG_dir -x fa -d ctg_depth.txt -sep

# depth file: tab separated, no header
contig_1    30
contig_2    5

=============================================================
'''


def get_color_list(color_num):

    if color_num <= 8:
        color_list_combined = ['#3787c0', '#39399f', '#ffb939', '#399f39', '#9f399f', '#fb694a', '#9f9f39', '#959595']

    elif 8 < color_num <= 16:
        color_list_combined = ['#2b7bba', '#89bedc', '#2e2e99', '#8a8acc', '#ffa500', '#ffc55c', '#2e992e', '#8acc8a', '#992e99', '#cc8acc', '#d52221', '#fc8161', '#99992e', '#cccc8a', '#5c5c5c', '#adadad']

    else:
        color_num_each = math.ceil(color_num/8) + 2

        color_list_1 = sns.color_palette('Blues',  n_colors=color_num_each).as_hex()
        color_list_2 = sns.light_palette('navy',   n_colors=color_num_each).as_hex()
        color_list_3 = sns.light_palette('orange', n_colors=color_num_each).as_hex()
        color_list_4 = sns.light_palette('green',  n_colors=color_num_each).as_hex()
        color_list_5 = sns.light_palette('purple', n_colors=color_num_each).as_hex()
        color_list_6 = sns.color_palette('Reds',   n_colors=color_num_each).as_hex()
        color_list_7 = sns.light_palette('olive',  n_colors=color_num_each).as_hex()
        color_list_8 = sns.color_palette('Greys', n_colors=color_num_each).as_hex()

        color_list_combined = []
        for color_list in [color_list_1, color_list_2, color_list_3, color_list_4, color_list_5, color_list_6, color_list_7, color_list_8]:
            for color in color_list[2:][::-1]:
                color_list_combined.append(color)

    color_list_to_return = random.sample(color_list_combined, color_num)

    color_list_to_return_sorted = []
    for color_to_return in color_list_combined:
        if color_to_return in color_list_to_return:
            color_list_to_return_sorted.append(color_to_return)

    return color_list_to_return_sorted


def just_plot(num_list_depth, num_list_gc, ctg_len_list, ctg_color_list, legend_elements, output_plot):

    ctg_len_list_adjusted = [l/2 for l in ctg_len_list]

    num_array_depth = np.array(num_list_depth)
    num_array_gc = np.array(num_list_gc)
    num_array_len = np.array(ctg_len_list_adjusted)

    plt.scatter(num_array_depth, num_array_gc, c=ctg_color_list, s=num_array_len, alpha=0.3, linewidths=0)
    #plt.scatter(num_array_depth, num_array_gc, c=ctg_color_list, s=5, alpha=0.6, linewidths=0)
    #plt.title('Number of genome: %s' % len(bin_file_list), fontsize=9)
    plt.title('Range of sequence length (Kbp): %s - %s' % (min(ctg_len_list), max(ctg_len_list)), fontsize=8)
    plt.xlabel("Depth (X)", size=9)
    plt.ylabel("GC (%)", size=9)
    plt.legend(handles=legend_elements, title='Genome', loc='upper left', prop={'size': 6}, bbox_to_anchor=(1, 1))
    #plt.legend(handles=legend_elements, loc='lower left', prop={'size': 6}, bbox_to_anchor=(1, 0.5))
    plt.tight_layout()
    plt.savefig(output_plot)
    plt.close()


def just_plot_backup(num_list_depth, num_list_gc, ctg_len_list, ctg_color_list, legend_elements, output_plot):

    num_array_depth = np.array(num_list_depth)
    num_array_gc = np.array(num_list_gc)
    num_array_len = np.array(ctg_len_list)
    fig, ax = plt.subplots()
    scatter = plt.scatter(num_array_depth, num_array_gc, c=ctg_color_list, s=num_array_len, alpha=0.3, linewidths=0)
    plt.title('Contig length (Kbp): %s - %s' % (min(ctg_len_list), max(ctg_len_list)), fontsize=9)
    plt.xlabel("Depth (X)", size=9)
    plt.ylabel("GC (%)", size=9)

    # add size legend
    # kw = dict(prop="sizes", num=6, color='grey', fmt="{x:.2f}", func=lambda s: np.sqrt(s/.3)/3)
    # handles, labels = scatter.legend_elements(**kw, alpha=0.5)
    # legend2 = ax.legend(handles, labels, loc="upper left", title="Size", prop={'size': 5}, bbox_to_anchor=(1, 1))
    # ax.add_artist(legend2)

    # add color legend
    plt.legend(handles=legend_elements, title='MAG', loc='upper left', prop={'size': 6}, bbox_to_anchor=(1, 0.5))

    # save and close
    plt.tight_layout()
    plt.savefig(output_plot)
    plt.close()


def Plot_MAG(argument_list):

    bin_folder = argument_list['i']
    bin_ext    = argument_list['x']
    depth_file = argument_list['d']
    plot_sep   = argument_list['sep']

    # read in depth info
    ctg_depth_dict = dict()
    for each_ctg in open(depth_file):
        each_ctg_split = each_ctg.strip().split('\t')
        ctg_depth_dict[each_ctg_split[0]] = float(each_ctg_split[1])

    # get mag file list
    bin_file_re = '%s/*.%s' % (bin_folder, bin_ext)
    bin_file_list = [os.path.basename(file_name) for file_name in glob.glob(bin_file_re)]

    if len(bin_file_list) == 0:
        print('No MAG detected, program exited')
        exit()

    color_list = get_color_list(len(bin_file_list))

    output_txt  = '%s_plot_GC_vs_depth.txt'     % bin_folder
    output_plot = '%s_plot_GC_vs_depth.pdf'     % bin_folder
    plot_folder = '%s_plot_GC_vs_depth'         % bin_folder
    txt_folder  = '%s_plot_GC_vs_depth/data'    % bin_folder
    if plot_sep is True:
        os.mkdir(plot_folder)
        os.mkdir(txt_folder)

    output_txt_handle = open(output_txt, 'w')
    output_txt_handle.write('Genome\tContig\tDepth\tGC\tLength(bp)\n')
    legend_elements = []
    num_list_depth = []
    num_list_gc = []
    ctg_len_list = []
    ctg_color_list = []
    index = 0
    for each_mag in bin_file_list:

        mag_no_ext        = each_mag[:-(len(bin_ext) + 1)]
        pwd_each_mag      = '%s/%s'     % (bin_folder, each_mag)
        pwd_each_mag_plot = '%s/%s.pdf' % (plot_folder, mag_no_ext)
        pwd_each_mag_txt  = '%s/%s.txt' % (txt_folder, mag_no_ext)

        if plot_sep is True:
            pwd_each_mag_txt_handle = open(pwd_each_mag_txt, 'w')
            pwd_each_mag_txt_handle.write('Contig\tDepth\tGC\tLength(bp)\n')
        legend_elements_sep = []
        num_list_depth_sep = []
        num_list_gc_sep = []
        ctg_len_list_sep = []
        ctg_color_list_sep = []
        for each_ctg in SeqIO.parse(pwd_each_mag, 'fasta'):
            ctg_depth = ctg_depth_dict[each_ctg.id]
            ctg_seq = str(each_ctg.seq).upper()
            ctg_gc = ((ctg_seq.count('G')) + (ctg_seq.count('C')))*100/len(ctg_seq)
            ctg_len_kbp = len(ctg_seq) / (1024)
            ctg_len_kbp = float("{0:.2f}".format(ctg_len_kbp))

            num_list_depth.append(ctg_depth)
            num_list_depth_sep.append(ctg_depth)

            num_list_gc.append(ctg_gc)
            num_list_gc_sep.append(ctg_gc)

            ctg_len_list.append(ctg_len_kbp)
            ctg_len_list_sep.append(ctg_len_kbp)

            ctg_color_list.append(color_list[index])
            ctg_color_list_sep.append(color_list[index])

            output_txt_handle.write('%s\t%s\t%s\t%s\t%s\n' % (mag_no_ext, each_ctg.id, ctg_depth, float("{0:.2f}".format(ctg_gc)), len(ctg_seq)))
            if plot_sep is True:
                pwd_each_mag_txt_handle.write('%s\t%s\t%s\t%s\n' % (each_ctg.id, ctg_depth, float("{0:.2f}".format(ctg_gc)), len(ctg_seq)))

        legend_elements.append(Patch(facecolor=color_list[index], edgecolor=color_list[index], label=mag_no_ext))
        legend_elements_sep.append(Patch(facecolor=color_list[index], edgecolor=color_list[index], label=mag_no_ext))
        if plot_sep is True:
            just_plot(num_list_depth_sep, num_list_gc_sep, ctg_len_list_sep, ctg_color_list_sep, legend_elements_sep, pwd_each_mag_plot)
            pwd_each_mag_txt_handle.close()
        index += 1

    just_plot(num_list_depth, num_list_gc, ctg_len_list, ctg_color_list, legend_elements, output_plot)
    output_txt_handle.close()


if __name__ == '__main__':

    Plot_MAG_parser = argparse.ArgumentParser(usage=Plot_MAG_parser_usage)
    Plot_MAG_parser.add_argument('-i',   required=True,                         help='MAG folder')
    Plot_MAG_parser.add_argument('-x',   required=True,                         help='file extension')
    Plot_MAG_parser.add_argument('-d',   required=True,                         help='contig depth')
    Plot_MAG_parser.add_argument('-sep', required=False, action='store_true',   help='get plot for individual MAGs')
    args = vars(Plot_MAG_parser.parse_args())
    Plot_MAG(args)



import os
import argparse
from time import sleep
import multiprocessing as mp


split_sam_usage = '''
==================== split_sam example command ====================

module load samtools/1.15
BioSAK split_sam -p single_ctg -i input.bam -r contig_1 -t 12
BioSAK split_sam -p multi_ctgs -i input.bam -r ctgs.txt -t 12

# Output files:
prefix.sam, prefix_sorted.bam and prefix_sorted.bam.bai

# ctgs.txt file format: one id per line, ">" excluded.

===================================================================
'''


def split_sam(args):

    sam_in        = args['i']
    input_ref_id  = args['r']
    output_prefix = args['p']
    thread_num    = args['t']

    # get input ref list
    input_ref_list = []
    if os.path.isfile(input_ref_id):
        for each_ref in open(input_ref_id):
            current_ref_id = each_ref.strip()
            input_ref_list.append(current_ref_id)
    else:
        input_ref_list = [input_ref_id]

    # get all header lines
    sam_all_header_lines_txt = '%s_all_header_lines.txt' % output_prefix
    get_sam_header_cmd = 'samtools view --header-only %s > %s' % (sam_in, sam_all_header_lines_txt)
    print(get_sam_header_cmd)
    os.system(get_sam_header_cmd)
    sleep(1)

    # define file name
    header_subset_txt    = '%s_header_lines.txt' % output_prefix
    sam_subset_no_header = '%s_no_header.sam'    % output_prefix
    subset_sam           = '%s.sam'              % output_prefix
    subset_bam           = '%s.bam'              % output_prefix
    subset_bam_sorted    = '%s_sorted.bam'       % output_prefix

    # get header line subset
    header_subset_txt_handle = open(header_subset_txt, 'w')
    for each_line in open(sam_all_header_lines_txt):
        if not each_line.startswith('@SQ'):
            header_subset_txt_handle.write(each_line)
        else:
            each_line_split = each_line.strip().split('\t')
            ctg_id = each_line_split[1][3:]
            if ctg_id in input_ref_list:
                header_subset_txt_handle.write(each_line)
    header_subset_txt_handle.close()

    # extract reads aligned to provided references
    if not os.path.isfile(input_ref_id):

        get_aln_cmd = 'samtools view %s %s -o %s' % (sam_in, input_ref_id, sam_subset_no_header)
        print(get_aln_cmd)
        os.system(get_aln_cmd)
        sleep(1)

    else:
        split_sam_wd = '%s_split_sam_wd' % output_prefix
        os.mkdir(split_sam_wd)

        subset_sam_cmd_list = []
        for each_ref in open(input_ref_id):
            current_ref_id = each_ref.strip()
            get_aln_cmd = 'samtools view %s %s -o %s/%s.sam' % (sam_in, current_ref_id, split_sam_wd, current_ref_id)
            subset_sam_cmd_list.append(get_aln_cmd)

        # extract alignments
        pool = mp.Pool(processes=thread_num)
        pool.map(os.system, subset_sam_cmd_list)
        pool.close()
        pool.join()

        combine_extract_aln_cmd = 'cat %s/*.sam > %s' % (split_sam_wd, sam_subset_no_header)
        os.system(combine_extract_aln_cmd)

        # remove tmp dir
        os.system('rm -r %s' % split_sam_wd)

    # combine header and aligned reads
    os.system('cat %s %s > %s' % (header_subset_txt, sam_subset_no_header, subset_sam))
    sleep(1)

    # remove tmp files
    os.system('rm %s' % sam_all_header_lines_txt)
    os.system('rm %s' % sam_subset_no_header)
    os.system('rm %s' % header_subset_txt)

    # sam to bam
    cmd_samtools_view  = 'samtools view -bS %s -o %s --threads %s' % (subset_sam, subset_bam, thread_num)
    print(cmd_samtools_view)
    os.system(cmd_samtools_view)
    sleep(1)

    # sort bam
    cmd_samtools_sort  = 'samtools sort %s -o %s --threads %s' % (subset_bam, subset_bam_sorted, thread_num)
    print(cmd_samtools_sort)
    os.system(cmd_samtools_sort)
    sleep(1)

    # index bam
    cmd_samtools_index = 'samtools index %s' % subset_bam_sorted
    print(cmd_samtools_index)
    os.system(cmd_samtools_index)
    sleep(1)

    print('Removing unsorted bam file')
    os.system('rm %s' % subset_bam)
    print('Done!')


if __name__ == '__main__':

    split_sam_parser = argparse.ArgumentParser(usage=split_sam_usage)
    split_sam_parser.add_argument('-p', required=True, help='output prefix')
    split_sam_parser.add_argument('-i', required=True, help='input sam/bam file')
    split_sam_parser.add_argument('-r', required=True, help='reference id')
    split_sam_parser.add_argument('-t', required=False, type=int, default=1, help='number of threads')
    args = vars(split_sam_parser.parse_args())
    split_sam(args)



import os
import random
import argparse
from Bio import SeqIO


SubsampleLongReads_usage = '''
====================== SubsampleLongReads example commands ======================

BioSAK SubsampleLongReads -i LongReads.fastq -s 5 -o op_dir_5 -fq
BioSAK SubsampleLongReads -i LongReads.fastq -s 1,5,10 -o op_dir_1-10 -fq
BioSAK SubsampleLongReads -i LongReads.fastq -s 1,25,50,75 -o op_dir_1-75 -fq

=================================================================================
'''


def SubsampleLongReads(args):

    seq_file        = args['i']
    subsample_str   = args['s']
    op_dir          = args['o']
    one_line        = args['oneline']
    in_fastq        = args['fq']

    # check if output dir exist
    if os.path.isdir(op_dir):
        print('The specified output directory already exist, program exited!')
        print(op_dir)
        exit()

    os.system('mkdir %s' % op_dir)

    seq_in_format = 'fasta'
    if in_fastq is True:
        seq_in_format = 'fastq'

    # read in read id
    seq_id_set = set()
    for seq_record in SeqIO.parse(seq_file, seq_in_format):
        seq_id = seq_record.id
        seq_id_set.add(seq_id)

    subsample_dict = dict()
    for subsample_pct in subsample_str.split(','):
        read_num_to_subset = round(len(seq_id_set)*float(subsample_pct)/100)
        reads_to_extract = random.sample([i for i in seq_id_set], read_num_to_subset)
        for each_read in reads_to_extract:
            if each_read not in subsample_dict:
                subsample_dict[each_read] = {subsample_pct}
            else:
                subsample_dict[each_read].add(subsample_pct)

    # subsampling
    for seq_record in SeqIO.parse(seq_file, seq_in_format):
        seq_id = seq_record.id
        if seq_id in subsample_dict:
            subsample_pct_set = subsample_dict[seq_id]
            for each_subsample_pct in subsample_pct_set:
                current_op_file = '%s/%s.%s' % (op_dir, each_subsample_pct, seq_in_format)
                with open (current_op_file, 'a') as current_op_file_handle:
                    if in_fastq is False:
                        if one_line is False:
                            SeqIO.write(seq_record, current_op_file_handle, 'fasta')
                        else:
                            SeqIO.write(seq_record, current_op_file_handle, 'fasta-2line')
                    else:
                        SeqIO.write(seq_record, current_op_file_handle, 'fastq')


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=SubsampleLongReads_usage)
    parser.add_argument('-i',       required=True,                       help='input sequence file')
    parser.add_argument('-s',       required=True,                       help='separate subsample rates (1-100) with comma, e.g. 1,5,10')
    parser.add_argument('-o',       required=True,                       help='output directory')
    parser.add_argument('-fq',      required=False, action="store_true", help='in fastq format, default: fa')
    parser.add_argument('-oneline', required=False, action="store_true", help='put sequence in single line')
    args = vars(parser.parse_args())
    SubsampleLongReads(args)
